import { r as registerInstance, c as createEvent, h, f as forceUpdate, g as getElement } from './p-597600b0.js';
import { o as loadAndUpdateConfigObj, p as getConfigFilePath } from './p-31525cd2.js';
import { S as SELECTED_DEVICE_SEARCH_PARAM, l as deviceToolMappingsPath, v as videoDirPath, j as imageDirPath, t as toolsInfo, m as deviceBlockDiagramDirPath, f as HOME_CONFIG } from './p-3ab399ac.js';

const adsDeviceSelectionCss = ":host{align-items:center;display:flex;height:42px}.selected-device{align-items:center;display:flex;gap:0.5em;margin:auto 1em;z-index:1}.selected-device .button{align-items:center;background-color:var(--color-ti-primary-white);border-radius:1em;color:var(--color-ti-primary-red);cursor:pointer;display:flex;font-size:0.9em;padding:5px 10px;text-transform:capitalize}.selected-device .label{color:var(--color-ti-primary-white);font-size:1.2rem;font-weight:bold}gc-widget-dialog{--gc-modal-opacity:0.8;z-index:9700;width:350px;height:500px;top:322px;left:593px;position:fixed}gc-widget-dialog .dialog-content{display:flex;gap:10px;margin:auto;flex-direction:column}gc-widget-dialog .dialog-content ads-control{--control-element-width:auto;--invalid-tooltip-disp:none}gc-widget-dialog .dialog-content gc-widget-icon{cursor:pointer}gc-widget-dialog .dialog-content .device-listbox{scroll-behavior:auto;min-height:265px;max-height:265px;min-width:200px;width:200px}gc-widget-dialog .dialog-content .current-device-container{color:var(--color-ti-primary-red);font-weight:bold}";

const AdsDeviceSelection = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.changeSelectedDevice = createEvent(this, "changeSelectedDevice", 7);
    this.isDialogOpen = false;
    /**
     * To copy device link to clipboard
     */
    this.copyDeviceLinkToClipboard = ev => {
      let locationUrl = new URL(window.location.href);
      let modifiedPathName = locationUrl.pathname.split("ver")[0];
      window.navigator.clipboard.writeText(locationUrl.origin +
        modifiedPathName +
        `?${SELECTED_DEVICE_SEARCH_PARAM}=${this.tempSelectedDeviceName}`);
      this.isShowCopiedIcon = true;
      setTimeout(() => {
        this.isShowCopiedIcon = false;
      }, 1000);
    };
    /**
     * To update the device selection in temporary variable when the user selects the value in the popup
     */
    this.updateTempSelectedDeviceName = ev => {
      if (this.isDialogOpen) {
        ev.stopPropagation();
        // this.deviceListSelectedVal = ev.detail.value;
        this.tempSelectedDeviceName = ev.detail.value;
      }
    };
    this.onDeviceChanged = (e) => {
      this.filterText = e.detail.value;
      let list = Object.keys(this.deviceToolMappings);
      let deviceList = document
        .querySelector("ads-main")
        .shadowRoot.querySelector("gc-widget-menubar")
        .querySelector("ads-device-selection")
        .shadowRoot.querySelector("gc-widget-dialog")
        .querySelector(".dialog-content")
        .querySelector("#device-listbox");
      {
        e.detail.value != undefined && e.detail.value != ""
          ? (this.filteredDevices = list.filter(name => name.toLowerCase().includes(e.detail.value) ||
            name.toUpperCase().includes(e.detail.value)))
          : "";
      }
      if (e.detail.value == undefined || e.detail.value == "") {
        this.filteredDevices = Object.keys(this.deviceToolMappings);
      }
      // this.tempSelectedDeviceName = this.filteredDevices[0];
      if ("selectedIndex" in deviceList && "selectedLabel" in deviceList) {
        deviceList.selectedLabel = this.filteredDevices[0];
        deviceList.selectedIndex = 0;
        // deviceList.refresh();
      }
    };
    /**
     * To emit an event to update the device selection in the GUI
     */
    this.updateDeviceSelection = () => {
      let deviceList = document
        .querySelector("ads-main")
        .shadowRoot.querySelector("gc-widget-menubar")
        .querySelector("ads-device-selection")
        .shadowRoot.querySelector("gc-widget-dialog")
        .querySelector(".dialog-content")
        .querySelector("#device-listbox");
      if ("labels" in deviceList && "selectedIndex" in deviceList) {
        this.tempSelectedDeviceName =
          deviceList.labels[`${deviceList.selectedIndex}`];
      }
      this.selectedDeviceName = this.tempSelectedDeviceName;
      this.closeDeviceSelectionPopup();
      this.changeSelectedDevice.emit({
        availableTools: this.deviceToolMappings[this.tempSelectedDeviceName],
        deviceName: this.tempSelectedDeviceName,
      });
    };
    /**
     * To show device selection popup
     */
    this.showDeviceSelectionPopup = () => {
      this.dialog.open();
      this.filterText = undefined;
    };
    /**
     * To close device selection popup
     */
    this.closeDeviceSelectionPopup = () => {
      // this.deviceListSelectedVal = this.selectedDeviceName;
      this.dialog.close();
      this.filterText = undefined;
    };
    // ---------- Helper methods ----------
    /**
     * To initialize the device tool mappings
     */
    this.initializeDeviceToolMappings = () => {
      return loadAndUpdateConfigObj.call(this, `deviceToolMappings`, deviceToolMappingsPath);
    };
    this.selectedDeviceName = undefined;
    this.deviceToolMappings = {};
    this.isShowCopiedIcon = false;
    this.tempSelectedDeviceName = undefined;
    this.filterText = undefined;
    this.filteredDevices = undefined;
    this.deviceListSelectedVal = undefined;
  }
  // ---------- Life cycle methods ----------
  async componentWillLoad() {
    await this.initializeDeviceToolMappings();
    this.filteredDevices = Object.keys(this.deviceToolMappings);
    this.tempSelectedDeviceName = this.filteredDevices[0];
  }
  componentDidLoad() {
    let locationUrl = new URL(window.location.href);
    if (Object.keys(this.deviceToolMappings).includes(locationUrl.href.split("=")[1])) {
      this.selectedDeviceName = locationUrl.href.split("=")[1];
      this.dialog.close();
    }
    else {
      this.selectedDeviceName = Object.keys(this.deviceToolMappings)[0];
      this.dialog.open();
    }
  }
  // ------- Watch handlers ------------
  onDeviceToolMappingsUpdate() {
    let deviceList = Object.keys(this.deviceToolMappings);
    if (deviceList.length) {
      let selectedDevice;
      const urlSelectedDevice = new URLSearchParams(window.location.search).get(SELECTED_DEVICE_SEARCH_PARAM);
      selectedDevice =
        Boolean(urlSelectedDevice) &&
          deviceList.find(deviceName => {
            return deviceName.toLowerCase() === urlSelectedDevice.toLowerCase();
          });
      if (!Boolean(selectedDevice)) {
        selectedDevice = deviceList[0];
      }
      this.changeSelectedDevice.emit({
        availableTools: this.deviceToolMappings[selectedDevice],
        deviceName: selectedDevice,
      });
    }
  }
  // ---------- Event handlers ----------
  onDialogOpened() {
    this.isDialogOpen = true;
    let deviceList = document
      .querySelector("ads-main")
      .shadowRoot.querySelector("gc-widget-menubar")
      .querySelector("ads-device-selection")
      .shadowRoot.querySelector("gc-widget-dialog")
      .querySelector(".dialog-content")
      .querySelector("#device-listbox");
    if ("selectedIndex" in deviceList && "selectedLabel" in deviceList) {
      deviceList.selectedLabel = this.filteredDevices[0];
      deviceList.selectedIndex = 0;
    }
    // this.deviceListSelectedVal = this.selectedDeviceName;
    // this.tempSelectedDeviceName = this.selectedDeviceName;
  }
  onDialogClosed() {
    this.isDialogOpen = false;
    // this.deviceListSelectedVal = this.selectedDeviceName;
  }
  render() {
    return [
      h("div", { class: "selected-device" }, h("span", { class: "label" }, this.selectedDeviceName), h("div", { class: "button", onClick: this.showDeviceSelectionPopup }, "Switch device")),
      h("gc-widget-dialog", { modal: true, ref: (el) => (this.dialog = el), heading: "Select Device", closeOnEsc: true, resizable: false }, h("div", { class: "dialog-content" }, h("div", { class: "current-device-container" }, h("span", null, "Current Device : "), this.selectedDeviceName.toUpperCase()), h("div", null, h("gc-widget-input-filter", { id: "search-device", placeholder: "Search devices by name", pattern: "^[^\\s]+$", hasClearIcon: true, hasSearchIcon: true, intermediateChanges: true, "onValue-changed": this.onDeviceChanged, value: this.filterText }), h("div", null, h("gc-widget-listbox", { class: "device-listbox", "initial-index": "0", labels: this.filteredDevices, "onSelected-label-changed": this.updateTempSelectedDeviceName, id: "device-listbox", selectedIndex: 0 })))), h("gc-widget-button", { slot: "action-bar", label: "Cancel", "button-type": "link", onClick: this.closeDeviceSelectionPopup }), h("gc-widget-button", { slot: "action-bar", label: "Update", "button-type": "link", disabled: this.filteredDevices.length == 0, onClick: this.updateDeviceSelection })),
    ];
  }
  static get watchers() { return {
    "deviceToolMappings": ["onDeviceToolMappingsUpdate"]
  }; }
};
AdsDeviceSelection.style = adsDeviceSelectionCss;

const adsHomeCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}:host{background-color:#f9f9f9;box-sizing:border-box;display:flex;font-size:max(14px, 1vw);gap:2em;height:100%;justify-content:space-between;overflow:auto;padding:4em;width:100%}:host .left-container{display:flex;flex-direction:column;min-width:max(30em, 400px);width:40%}:host .right-container{display:flex;flex-direction:column;min-width:max(100% - 32em, 600px);width:60%}.collaterals .collateral{align-items:center;background-color:var(--color-ti-primary-white);border-radius:0.2em;box-shadow:0 0 0.2em 0 rgba(216, 216, 216, 0.5);display:flex;font-size:0.8em;font-weight:bold;justify-content:center;padding:15px 0px;text-decoration:unset}.collaterals .collateral.highlight{background-color:rgba(204, 0, 0, 0.7)}.collaterals .collateral,.collaterals .collateral:active,.collaterals .collateral:visited{color:var(--color-ti-secondary-blue-light)}.collaterals .collateral.highlight,.collaterals .collateral:active.highlight,.collaterals .collateral:visited.highlight{color:var(--color-ti-primary-white)}.collaterals .collateral.disabled{color:var(--color-ti-secondary-grey);pointer-events:none}.collaterals .title{color:var(--color-ti-text-grey-dark);font-size:1em;font-weight:bold;margin-bottom:1em;margin-top:2em;text-transform:uppercase}.collaterals .wrapper{display:grid;flex-wrap:wrap;gap:1.2em;grid-template-columns:repeat(5, 1fr)}.device-details .desc{color:var(--color-ti-text-grey);font-size:0.8em;line-height:1.5em;margin-bottom:2em;text-align:justify}.device-details .name{color:var(--color-ti-text-grey-dark);font-size:1.5em;font-weight:bold;margin-bottom:0.7em}.image{display:flex;justify-content:center;min-height:max(300px, 25em)}.image img{max-height:100%;max-width:100%}.placeholder{align-items:center;background-color:#f5f5f5;border:0.1em dashed #979797;border-radius:0.2em;box-sizing:border-box;color:#acacac;display:flex;font-size:0.8em;font-weight:bold;height:100%;justify-content:center;text-transform:uppercase;width:100%}.tools{display:flex;flex-direction:column;height:100%;overflow:auto}.tools .page-links-wrapper{height:100%;overflow:auto}.tools .page-link{border:1px solid var(--color-ti-border-grey-dark);cursor:pointer;display:flex;margin:15px 15px 0px 0px;padding:15px}.tools .page-link .details{padding:0px 8px;width:100%}.tools .page-link .details .desc{color:var(--color-ti-text-grey);font-size:0.8em;line-height:1.5em;text-align:justify}.tools .page-link .details .name{color:var(--color-ti-text-grey-dark);font-size:0.9em;font-weight:500}.tools .page-link .forward-icon{padding-right:10px;--gc-color:var(--color-ti-secondary-blue-dark)}.tools .page-link gc-widget-icon{margin:0}.tools .page-link .page-icon{margin-bottom:auto;--gc-color:var(--color-ti-primary-red)}.tools .title{align-items:center;color:var(--color-ti-text-blue);display:flex;font-size:1em;font-weight:bold;margin-bottom:10px}.tools .title gc-widget-icon{cursor:help;--gc-color:var(--color-ti-text-blue)}.video{height:100%;max-height:max(210px, 15em);min-height:max(140px, 10em)}.video .title{color:var(--color-ti-text-grey-dark);font-size:1em;font-weight:bold;margin-bottom:1em;margin-top:1em;text-transform:uppercase}.video .wrapper{height:calc(100% - 4em);max-width:max(300px, 25em);min-width:max(300px, 20em)}";

const AdsHome = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.setLoaderState = createEvent(this, "setLoaderState", 7);
    this.showToast = createEvent(this, "showToast", 7);
    this.navigatePage = createEvent(this, "navigatePage", 7);
    // ---------- Helper methods ----------
    /**
     * To update the configuration objects w.r.t the selected device
     */
    this.updateConfigurations = async () => {
      if (this.selectedDeviceName === "") {
        return;
      }
      this.setLoaderState.emit({ show: true });
      // Home info
      await loadAndUpdateConfigObj.call(this, `homeInfo`, getConfigFilePath(HOME_CONFIG, this.selectedDeviceName));
      setTimeout(() => {
        forceUpdate(this.host);
        this.setLoaderState.emit({ show: false });
        this.showToast.emit({
          message: `${this.selectedDeviceName} Device is selected`,
        });
      }, 1000);
    };
    /**
     * To navigate tab in gc-widget-tab-container
     * @param tabIndex tab index
     */
    this.navigateTab = tabIndex => {
      this.navigatePage.emit({ tabIndex });
    };
    this.availableTools = undefined;
    this.selectedDeviceName = undefined;
    this.homeInfo = undefined;
  }
  // ---------- Watch Handlers ----------
  onSelectedDeviceNameChanged() {
    this.updateConfigurations();
  }
  render() {
    return this.homeInfo
      ? [
        h("div", { class: "left-container" }, h("div", { class: "device-details" }, h("div", { class: "name" }, this.homeInfo.deviceName), h("div", { class: "desc" }, this.homeInfo.deviceDesc)), h("div", { class: "tools" }, h("div", { class: "title" }, h("span", null, "GET STARTED"), h("span", { class: "tool-info", id: "tool-nav-info" }, h("gc-widget-icon", { appearance: "custom", icon: "filled:action:info_outline", size: "s" })), h("gc-widget-tooltip", { "anchor-id": "tool-nav-info", position: "top", text: "Click on any tool below to view its calculator page." })), h("div", { class: "page-links-wrapper" }, this.availableTools.map((tool, index) => {
          const iconName = tool.startsWith("crc") ? "crc" : tool;
          return (h("div", { class: "page-link", onClick: () => {
              this.navigateTab(index);
            } }, h("gc-widget-icon", { appearance: "custom", class: "page-icon", icon: `home_pageIcons:red_${iconName}`, path: imageDirPath, size: "l" }), h("div", { class: "details" }, h("div", { class: "name" }, toolsInfo[tool].name), h("div", { class: "desc" }, " ", toolsInfo[tool].desc, " ")), h("gc-widget-icon", { appearance: "custom", class: "forward-icon", icon: `filled:navigation:arrow_forward`, size: "s" })));
        }))), this.homeInfo.video.fileName !== "" && (h("div", { class: "video" }, h("div", { class: "title" }, "Video tutorial"), h("div", { class: "wrapper" }, h("ads-video", { videoSrc: `${videoDirPath}${this.homeInfo.video.fileName}` }))))),
        h("div", { class: "right-container" }, h("div", { class: "image" }, this.homeInfo.image.fileName !== "" ? (h("img", { src: `${deviceBlockDiagramDirPath}${this.homeInfo.image.fileName}`, alt: "Image" })) : (h("div", { class: "placeholder" }, h("span", null, "Placeholder for image")))), h("div", { class: "collaterals" }, h("div", { class: "title" }, "Information links"), h("div", { class: "wrapper" }, this.homeInfo.collaterals.map(link => {
          return (h("a", { class: "collateral" +
              (link.url === "" ? " disabled" : "") +
              (link.isHighlight ? " highlight" : ""), href: link.url, target: "_blank" }, link.label));
        })))),
      ]
      : [h("div", null)];
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "selectedDeviceName": ["onSelectedDeviceNameChanged"]
  }; }
};
AdsHome.style = adsHomeCss;

const adsToastCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}:host{bottom:40px;font-family:Roboto;position:absolute;right:30px;z-index:1}.toast-container{align-items:center;background-color:var(--color-ti-text-black);box-shadow:0px 0px 2px 0px var(--color-ti-text-black);color:var(--color-ti-primary-white);display:flex;padding:10px 20px;text-align:center}.toast-container .done-icon{cursor:pointer;margin:10px 10px 10px 0px;--gc-color:var(--color-ti-primary-white)}.toast-container .done-icon:hover{background-color:var(--color-ti-primary-white);--gc-color:var(--color-ti-primary-black)}";

const AdsToast = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.message = undefined;
    this.timeout = 5;
    this.isShow = false;
    this.timoutHandler = null;
  }
  // ---------- Watch handlers ----------
  showToastMessage() {
    switch (true) {
      case this.message.startsWith("CRC for"):
        this.message = "CRC is computed";
        break;
      case this.message.startsWith("ADC Settings"):
        this.message = "Filter Response and Time to Conversion is updated";
        break;
      case this.message.startsWith("Programmable Delay"):
        this.message = "Time to Conversion is updated";
        break;
      case this.message.startsWith("Import file"):
        this.message = "File is imported successfully";
        break;
      case this.message.startsWith("Reset default"):
        this.message = "Register values are set to default values";
        break;
      case this.message.startsWith("Reset Tue"):
        this.message = "All the control values are set to default values";
        break;
    }
    if (Boolean(this.message)) {
      if (this.timoutHandler) {
        // clear timeout if already exists
        clearTimeout(this.timoutHandler);
      }
      this.show();
      this.timoutHandler = setTimeout(() => {
        this.hide();
        this.timoutHandler = null;
      }, this.timeout * 1000);
    }
  }
  // ---------- Helper methods ----------
  /**
   * Show toast
   */
  show() {
    this.isShow = true;
  }
  /**
   * Hide toast
   */
  hide() {
    this.isShow = false;
  }
  // ---------- DOM ----------
  render() {
    return [
      this.isShow && (h("div", { class: "toast-container" }, h("gc-widget-icon", { appearance: "custom", class: "done-icon", icon: "filled:action:done", size: "s" }), h("span", { class: "toast-header" }, this.message))),
    ];
  }
  static get watchers() { return {
    "message": ["showToastMessage"]
  }; }
};
AdsToast.style = adsToastCss;

export { AdsDeviceSelection as ads_device_selection, AdsHome as ads_home, AdsToast as ads_toast };

//# sourceMappingURL=p-32b76337.entry.js.map