import { r as registerInstance, c as createEvent, h, g as getElement } from './p-597600b0.js';
import { u as updateHypentoUnderscore } from './p-d57b9fa3.js';
import { bindingRegistry } from '../../components/@ti/gc-core-databind/lib/CoreDatabind';

const adsRegisterMapCss = "ads-popup::part(popup-container){height:75%;width:75%}.page-header{box-sizing:content-box;display:flex;height:28px;justify-content:space-between;padding-bottom:5px}.page-header .page-title{color:var(--color-ti-text-grey);font-size:24px}.page-content{display:flex;height:calc(100% - 33px)}.page-content .left-section,.page-content .right-section{display:flex;flex-direction:column;gap:2%;height:100%;min-height:670px;padding:2%}.page-content .left-section{background-color:var(--color-ti-primary-white);min-width:540px;width:40%}.page-content .right-section{background-color:#f2f8fa;min-width:720px;width:60%}.popup-btn{align-items:center;cursor:pointer;display:flex;text-decoration:underline}.popup-content{display:flex;flex-direction:column}.popup-content img{margin:auto;max-height:calc(100% - 100px);max-width:calc(100% - 100px)}.popup-content .title{background-color:var(--color-ti-indicator-grey);font-size:1.5rem;font-weight:bold;height:40px;margin:0 auto;padding:5px;text-align:center;width:50%}.section-title{color:var(--color-ti-text-blue);font-size:18px}.section-tabs,.section-outputs{display:flex;flex-direction:column;height:100%;overflow:auto;width:100%}.section-outputs ads-tabs{--selected-tab-color:var(--color-ti-primary-blue-dark)}.section-outputs .tab-content{display:flex;flex-direction:column}.tab-content{height:max-content;overflow:hidden auto;width:100%}.tab-content ads-control{--control-label-width:50%}.tab-content .group-title{color:var(--color-ti-primary-red);font-size:1.2em;margin-top:5%}.page-header .page-controls{display:flex}.forward-icon{--gc-color:white}.export-button{color:white;font-weight:bold;font-size:14px;background-color:var(--color-ti-text-blue);border:0px solid;width:100px;margin-right:5px;cursor:pointer}.export-button:hover{background-color:var(--color-ti-secondary-blue-light)}.button-label{position:relative;bottom:5px;padding-right:10px}.export-button:active{box-shadow:0px 0px 5px 1px var(--color-ti-text-blue)}gc-widget-dialog{--gc-modal-opacity:0.8;z-index:-1}gc-widget-dialog .dialog-content{width:334px;padding:25px}gc-widget-input-filter{width:99%}:host{background-color:var(--color-ti-bg-grey-dark);display:flex;flex-direction:column;height:100%;overflow:auto;padding:2%;width:100%}.page-header{min-width:1215px}.page-content{flex-direction:column;background-color:#ffffff;min-width:1215px}.section-content{display:flex;height:calc(100% - 38px)}gc-custom-register-grid{width:60%;min-width:720px;min-height:0px}gc-custom-register-bitfield-viewer{width:40%;min-width:480px;min-height:0px}";

const AdsRegisterMap = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.showToast = createEvent(this, "showToast", 7);
    this.onSelectedFieldIndexChanged = (e) => {
      this.selectedFieldIndex = e.detail.value;
    };
    this.onSelectedRegisterChanged = (e) => {
      this.selectedRegister = e.detail.value;
    };
    this.onFilterTextChanged = (e) => {
      this.filterText = e.detail.value;
    };
    this._exportConfigFile = (e) => {
      let selectedDevice = updateHypentoUnderscore(this.selectedDevice);
      let temp1 = bindingRegistry.models["entries"]();
      let data = `ADDRESS      VALUE      NAME\n`;
      data += "\n";
      let tempdata = Array.from(temp1, ([name, value]) => ({ name, value }));
      const index = tempdata.findIndex(object => {
        return object.name === selectedDevice;
      });
      let valArray = tempdata[index].value.modelBindings;
      let currentData = Array.from(valArray, ([name, value]) => ({
        name,
        value,
      }));
      let arraylength = tempdata[index].value.registerJsonData.regblocks.length;
      for (let i = 0; i < arraylength; i++) {
        tempdata[index].value.registerJsonData.regblocks[i].registers.forEach(addressElement => {
          currentData.forEach(element => {
            if (addressElement.id === element.name) {
              if (!element.name.startsWith("$")) {
                let addressval = parseInt(addressElement.addr.toString(16), 16)
                  .toString(16)
                  .toUpperCase()
                  .padStart(2, 0);
                let length = addressElement.size;
                let size;
                switch (length) {
                  case 8:
                    size = 2;
                    break;
                  case 16:
                    size = 4;
                    break;
                  default:
                    size = 2;
                    break;
                }
                let hexValue = parseInt(element.value.cachedValue.toString(16), 16)
                  .toString(16)
                  .toUpperCase()
                  .padStart(size, "0");
                data += `0x${addressval}         0x${hexValue}       ${addressElement.name}\n`;
              }
            }
          });
        });
      }
      let encodedData = "data:text/plain;charset=utf-8," + encodeURIComponent(`${data}`);
      let hiddenElement = document.createElement("a");
      hiddenElement.style.display = "none";
      hiddenElement.href = encodedData;
      hiddenElement.download = `${selectedDevice}_RegisterDetails.txt`;
      hiddenElement.click();
      hiddenElement.remove();
    };
    this._importConfigFile = (e) => {
      this.importElement.click();
    };
    this._populateConfigFileValues = ev => {
      try {
        let writeData = true;
        let reader = new FileReader();
        let file = ev.target.files[0];
        if (file.name.endsWith(".txt")) {
          reader.readAsText(file);
          reader.onload = () => {
            try {
              let addressIndex = reader.result
                .split("\n")[0]
                .split(" ")
                .filter(el => {
                return el != "";
              })
                .indexOf("ADDRESS");
              let valueIndex = reader.result
                .split("\n")[0]
                .split(" ")
                .filter(el => {
                return el != "";
              })
                .indexOf("VALUE");
              let nameIndex = reader.result
                .split("\n")[0]
                .split(" ")
                .filter(el => {
                return el != "";
              })
                .indexOf("NAME");
              if (addressIndex != -1 && valueIndex != -1 && nameIndex != -1) {
                let selectedDevice = updateHypentoUnderscore(this.selectedDevice);
                let temp1 = bindingRegistry.models["entries"]();
                let tempdata = Array.from(temp1, ([name, value]) => ({
                  name,
                  value,
                }));
                const index = tempdata.findIndex(object => {
                  return object.name === selectedDevice;
                });
                let arraylength = tempdata[index].value.registerJsonData.regblocks.length;
                let j = 0;
                for (let i = 0; i < arraylength; i++) {
                  tempdata[index].value.registerJsonData.regblocks[i].registers.forEach(addressElement => {
                    let addressval = parseInt(addressElement.addr.toString(16), 16)
                      .toString(16)
                      .toUpperCase()
                      .padStart(2, 0);
                    let length = addressElement.size;
                    let size;
                    switch (length) {
                      case 8:
                        size = 2;
                        break;
                      case 16:
                        size = 4;
                        break;
                      default:
                        size = 2;
                        break;
                    }
                    let arrayData = reader.result
                      .split("\n")
                      .slice(2)[j].split(/[ \t]+/)
                      .filter(el => {
                      return el != "" && el != "\t";
                    });
                    if (arrayData.length == 3) {
                      if (arrayData[addressIndex] == `0x${addressval}` &&
                        arrayData[valueIndex].length == size + 2 &&
                        arrayData[nameIndex] == addressElement.name) {
                        j++;
                      }
                      else {
                        writeData = false;
                        if (arrayData[addressIndex] != `0x${addressval}`) {
                          throw new Error(`Register Address "0x${addressval}" is missing or invalid in the selected file`);
                        }
                        if (arrayData[nameIndex] != addressElement.name) {
                          throw new Error(`Invalid Register Name "${arrayData[nameIndex]}" in the selected file, expected register name is "${addressElement.name}" for address "0x${addressval}"`);
                        }
                        if (arrayData[valueIndex].length != size + 2) {
                          throw new Error(`Invalid Register Value for "${addressElement.name}" in the selected file, expected size is "${size}"bytes`);
                        }
                      }
                    }
                    else {
                      writeData = false;
                      throw new Error(`Invalid Data "${reader.result.split("\n").slice(2)[j]}" in the selected file`);
                    }
                  });
                }
                if (writeData) {
                  j = 0;
                  let valArray = tempdata[index].value.modelBindings;
                  let currentData = Array.from(valArray, ([name, value]) => ({
                    name,
                    value,
                  }));
                  for (let i = 0; i < arraylength; i++) {
                    tempdata[index].value.registerJsonData.regblocks[i].registers.forEach(addressElement => {
                      let arrayData = reader.result
                        .split("\n")
                        .slice(2)[j].split(" ")
                        .filter(el => {
                        return el != "";
                      });
                      let keyVal = currentData.findIndex(object => {
                        return object.name === addressElement.id;
                      });
                      currentData[keyVal].value.cachedValue = parseInt(arrayData[valueIndex]);
                      if (this.selectedRegister == addressElement.id) {
                        let bitfieldViewer = this.host.shadowRoot.querySelector("gc-custom-register-bitfield-viewer");
                        bitfieldViewer.value = parseInt(arrayData[valueIndex]);
                      }
                      j++;
                    });
                  }
                  let registerGrid = this.host.shadowRoot.querySelector("gc-custom-register-grid");
                  registerGrid.redrawGrid();
                  let today = new Date();
                  let time = today.getHours().toString().padStart(2, 0) +
                    "-" +
                    today.getMinutes().toString().padStart(2, 0) +
                    "-" +
                    today.getSeconds().toString().padStart(2, 0);
                  `${this.showToast.emit({
                    message: `Import file ${time}`,
                  })}`;
                }
              }
              else {
                throw new Error("Header is missing or invalid in the selected file");
              }
            }
            catch (error) {
              console.log("%c" + error, "color:red;font-size:18px;");
            }
          };
        }
        else {
          throw new Error("Invalid File Type. Only text files are supported");
        }
      }
      catch (error) {
        console.log("%c" + error, "color:red;font-size:18px;");
      }
      finally {
        this.importElement.value = "";
      }
    };
    this.showResetConfirmationPopup = () => {
      this.dialog.open();
    };
    this.closeResetConfirmationPopup = () => {
      this.dialog.close();
    };
    this._resetDefaults = (e) => {
      let selectedDevice = updateHypentoUnderscore(this.selectedDevice);
      let temp1 = bindingRegistry.models["entries"]();
      let tempdata = Array.from(temp1, ([name, value]) => ({ name, value }));
      const index = tempdata.findIndex(object => {
        return object.name === selectedDevice;
      });
      let valArray = tempdata[index].value.modelBindings;
      let currentData = Array.from(valArray, ([name, value]) => ({
        name,
        value,
      }));
      currentData.forEach(element => {
        if (!element.name.startsWith("$")) {
          element.value.cachedValue = element.value.defaultValue;
          if (this.selectedRegister == element.name) {
            let bitfieldViewer = this.host.shadowRoot.querySelector("gc-custom-register-bitfield-viewer");
            bitfieldViewer.value = element.value.cachedValue;
          }
        }
      });
      let registerGrid = this.host.shadowRoot.querySelector("gc-custom-register-grid");
      registerGrid.redrawGrid();
      this.closeResetConfirmationPopup();
      let today = new Date();
      let time = today.getHours().toString().padStart(2, 0) +
        "-" +
        today.getMinutes().toString().padStart(2, 0) +
        "-" +
        today.getSeconds().toString().padStart(2, 0);
      `${this.showToast.emit({
        message: `Reset default ${time}`,
      })}`;
    };
    this.activeRegisterModel = undefined;
    this.activeDeviceRegisterInfo = undefined;
    this.selectedRegister = undefined;
    this.selectedFieldIndex = -1;
    this.activeDeviceName = "";
    this.filterText = "";
    this.showDetails = undefined;
    this.busyProgress = undefined;
    this.selectedDevice = undefined;
  }
  render() {
    var _a, _b, _c, _d;
    const selectedRegister = (_a = this.activeRegisterModel) === null || _a === void 0 ? void 0 : _a.getRegisterInfo(this.selectedRegister);
    const selectedField = (_b = selectedRegister === null || selectedRegister === void 0 ? void 0 : selectedRegister.fields) === null || _b === void 0 ? void 0 : _b[this.selectedFieldIndex];
    const detailsRegister = this.showDetails
      ? (_c = this.activeRegisterModel) === null || _c === void 0 ? void 0 : _c.getRegisterInfo(this.showDetails)
      : undefined;
    return [
      h("div", { class: "page-header" }, h("div", { class: "page-title" }, "Register Map"), h("div", { class: "page-controls" }, h("button", { class: "export-button", onClick: this.showResetConfirmationPopup }, h("svg", { id: "restore-factory-defaults", width: "25px", height: "22px", viewBox: "0 0 20 14", version: "1.1", xmlns: "http://www.w3.org/2000/svg", "xmlns:xlink": "http://www.w3.org/1999/xlink" }, h("g", { id: "Icons", stroke: "none", "stroke-width": "1", fill: "none", "fill-rule": "evenodd" }, h("g", { transform: "translate(-827.000000, -60.000000)", id: "RESTORE-DEFAULTS" }, h("g", { transform: "translate(827.000000, 60.000000)" }, h("rect", { id: "Rectangle-Copy-19", x: "0", y: "0", width: "16", height: "16" }), h("g", { id: "RESTORE-DEFAULTS_GRAY" }, h("rect", { id: "Rectangle-Copy-26", x: "0", y: "0", width: "16", height: "16" }), h("path", { d: "M11.5303315,4.46875 C10.6235147,3.5625 9.37898687,3 7.99687305,3 C5.2326454,3 3,5.2375 3,8 C3,10.7625 5.2326454,13 7.99687305,13 C10.329581,13 12.2745466,11.40625 12.8311445,9.25 L11.5303315,9.25 C11.0175109,10.70625 9.62914321,11.75 7.99687305,11.75 C5.92682927,11.75 4.24452783,10.06875 4.24452783,8 C4.24452783,5.93125 5.92682927,4.25 7.99687305,4.25 C9.03502189,4.25 9.96060038,4.68125 10.6360225,5.3625 L8.62226391,7.375 L13,7.375 L13,3 L11.5303315,4.46875 Z", id: "Path-Copy-4", fill: "#ffffff", "fill-rule": "nonzero" })))))), h("span", { class: "button-label" }, "Reset")), h("button", { class: "export-button", onClick: this._importConfigFile }, h("svg", { id: "import", width: "25px", height: "22px", viewBox: "0 0 20 16", version: "1.1", xmlns: "http://www.w3.org/2000/svg", "xmlns:xlink": "http://www.w3.org/1999/xlink" }, h("g", { id: "Page-1", stroke: "none", "stroke-width": "1", fill: "none", "fill-rule": "evenodd" }, h("g", { id: "Icons", transform: "translate(-595.000000, -60.000000)" }, h("g", { id: "IMPORT", transform: "translate(595.000000, 60.000000)" }, h("rect", { id: "Rectangle-Copy-16", x: "0", y: "0", width: "16", height: "16" }), h("g", { id: "Group-9-Copy", transform: "translate(3.000000, 3.000000)", stroke: "#ffffff", "stroke-linecap": "round", "stroke-linejoin": "round", "stroke-width": "1.5" }, h("path", { d: "M10,6.66666667 L10,8.88888889 C10,9.50253861 9.50253861,10 8.88888889,10 L1.11111111,10 C0.497461389,10 0,9.50253861 0,8.88888889 L0,6.66666667", id: "Path" }), h("polyline", { id: "Path", points: "2.22222222 3.88888889 5 6.66666667 7.77777778 3.88888889" }), h("line", { x1: "5", y1: "6.66666667", x2: "5", y2: "0", id: "Path" })))))), h("span", { class: "button-label" }, "Import")), h("button", { class: "export-button", onClick: this._exportConfigFile }, h("svg", { class: "forward-icon", id: "export", width: "25px", height: "22px", viewBox: "0 0 20 16", version: "1.1", xmlns: "http://www.w3.org/2000/svg", "xmlns:xlink": "http://www.w3.org/1999/xlink" }, h("g", { id: "Page-1", stroke: "none", "stroke-width": "1", fill: "none", "fill-rule": "evenodd" }, h("g", { id: "Icons", transform: "translate(-667.000000, -60.000000)" }, h("g", { id: "EXPORT", transform: "translate(667.000000, 60.000000)" }, h("rect", { id: "Rectangle-Copy-17", x: "0", y: "0", width: "16", height: "16" }), h("g", { id: "Group-9-Copy", transform: "translate(3.000000, 3.000000)", stroke: "#ffffff", "stroke-linecap": "round", "stroke-linejoin": "round", "stroke-width": "1.5" }, h("path", { d: "M10,6.66666667 L10,8.88888889 C10,9.50253861 9.50253861,10 8.88888889,10 L1.11111111,10 C0.497461389,10 0,9.50253861 0,8.88888889 L0,6.66666667", id: "Path" }), h("g", { id: "Group", transform: "translate(5.000000, 3.333333) rotate(180.000000) translate(-5.000000, -3.333333) translate(2.222222, 0.000000)" }, h("polyline", { id: "Path", points: "-1.75462171e-12 3.88888889 2.77777778 6.66666667 5.55555556 3.88888889" }), h("line", { x1: "2.77777778", y1: "6.66666667", x2: "2.77777778", y2: "0", id: "Path" }))))))), h("span", { class: "button-label" }, "Export")))),
      h("div", { class: "page-content" }, h("gc-widget-input-filter", { id: "search", placeholder: "Search registers by name", class: "stretch", hasClearIcon: true, hasSearchIcon: true, intermediateChanges: true, "onValue-changed": this.onFilterTextChanged }), h("div", { class: "section-content" }, h("gc-custom-register-grid", { "register-model-id": updateHypentoUnderscore(this.selectedDevice), "selected-field": selectedField === null || selectedField === void 0 ? void 0 : selectedField.id, "on-selected-register-changed": this.onSelectedRegisterChanged, filterText: detailsRegister
          ? this.showDetails
          : (_d = this.filterText) === null || _d === void 0 ? void 0 : _d.toUpperCase() }), h("gc-custom-register-bitfield-viewer", { id: "bitFieldViewer", heading: "Register Details", "register-model-id": updateHypentoUnderscore(this.selectedDevice), "register-name": this.selectedRegister, "on-selected-field-index-changed": this.onSelectedFieldIndexChanged }))),
      h("gc-widget-dialog", { modal: true, ref: (el) => (this.dialog = el), heading: "Reset Default", closeOnEsc: true, resizable: false }, h("div", { class: "dialog-content" }, "This will reset all the registers to their default values. Do you want to continue?"), h("gc-widget-button", { slot: "action-bar", label: "No", "button-type": "link", onClick: this.closeResetConfirmationPopup }), h("gc-widget-button", { slot: "action-bar", label: "Yes", "button-type": "link", onClick: this._resetDefaults })),
      h("input", { type: "file", accept: ".txt", style: { display: "none" }, ref: el => (this.importElement = el), onChange: e => this._populateConfigFileValues(e) }),
    ];
  }
  get host() { return getElement(this); }
};
AdsRegisterMap.style = adsRegisterMapCss;

export { AdsRegisterMap as ads_register_map };

//# sourceMappingURL=p-5a081d24.entry.js.map