import * as baseCalculation from "./cm-range-calculation.js";

let DGAIN;

export function calculate_AVDD(args) {
  return baseCalculation.calculate_AVDD(args);
}

export function calculate_AVSS(args) {
  return baseCalculation.calculate_AVSS(args);
}

export function calculate_PGA_GAIN(args) {
  return baseCalculation.calculate_PGA_GAIN(args);
}

export function calculate_VREFN(args) {
  return baseCalculation.calculate_VREFN(args);
}

export function calculate_VREFP(args) {
  return baseCalculation.calculate_VREFP(args);
}

export function calculate_AINP(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let pga_gain = baseCalculation.compute_pga_gain(args);
  let absMinVal = baseCalculation.compute_AINx_ABS_MIN(args);
  let absMaxVal = baseCalculation.compute_AINx_ABS_MAX(args);

  if (args.input_mode.value === "Differential/CM Voltages") {
    return {
      absMinValue: "",
      absMaxValue: "",
      isDisabled: true,
      minValue: undefined,
      maxValue: undefined,
      removeTooltip: true,
      value: args.vin_cm.value + args.vin_diff.value / 2,
    };
  }
  let AinpVal =
    args.parameterConfigData[args.selectedDevice].AINP != undefined
      ? args.parameterConfigData[args.selectedDevice].AINP
      : args.ainp.value;
  let val = args.ainp.value != undefined ? args.ainp.value : AinpVal;

  let minVal;
  let maxVal;
  if (configData && configData.NEG_SWING_TO_RAIL != undefined) {
    if (configData.NEG_SWING_TO_RAIL.bufferEnabled != undefined) {
      minVal = args.pga_gain.value
        ? args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferEnabled)
        : args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.NEG_SWING_TO_RAIL.bufferDisabled == undefined) {
      minVal =
        args.avss.value + parseFloat(configData.NEG_SWING_TO_RAIL[pga_gain]);
    }
  }
  if (configData && configData.POS_SWING_TO_RAIL != undefined) {
    if (configData.POS_SWING_TO_RAIL.bufferEnabled != undefined) {
      maxVal = args.pga_gain.value
        ? args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferEnabled)
        : args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.POS_SWING_TO_RAIL.bufferDisabled == undefined) {
      maxVal =
        args.avdd.value - parseFloat(configData.POS_SWING_TO_RAIL[pga_gain]);
    }
  }

  return {
    absMinValue: absMinVal,
    absMaxValue: absMaxVal,
    minValue: minVal,
    removeTooltip: false,
    maxValue: maxVal,
    isDisabled: false,
    value: val,
  };
}

export function calculate_AINN(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let pga_gain = baseCalculation.compute_pga_gain(args);
  let absMinVal = baseCalculation.compute_AINx_ABS_MIN(args);
  let absMaxVal = baseCalculation.compute_AINx_ABS_MAX(args);
  if (args.input_mode.value === "Differential/CM Voltages") {
    return {
      absMinValue: "",
      absMaxValue: "",
      removeTooltip: true,
      isDisabled: true,
      minValue: undefined,
      maxValue: undefined,
      value: args.vin_cm.value - args.vin_diff.value / 2,
    };
  }
  let AinnVal =
    args.parameterConfigData[args.selectedDevice].AINN != undefined
      ? args.parameterConfigData[args.selectedDevice].AINN
      : args.ainn.value;
  let val = args.ainn.value != undefined ? args.ainn.value : AinnVal;
  let minVal;
  let maxVal;
  if (configData && configData.NEG_SWING_TO_RAIL != undefined) {
    if (configData.NEG_SWING_TO_RAIL.bufferEnabled != undefined) {
      minVal = args.pga_gain.value
        ? args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferEnabled)
        : args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.NEG_SWING_TO_RAIL.bufferDisabled == undefined) {
      minVal =
        args.avss.value + parseFloat(configData.NEG_SWING_TO_RAIL[pga_gain]);
    }
  }
  if (configData && configData.POS_SWING_TO_RAIL != undefined) {
    if (configData.POS_SWING_TO_RAIL.bufferEnabled != undefined) {
      maxVal = args.pga_gain.value
        ? args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferEnabled)
        : args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.POS_SWING_TO_RAIL.bufferDisabled == undefined) {
      maxVal =
        args.avdd.value - parseFloat(configData.POS_SWING_TO_RAIL[pga_gain]);
    }
  }
  let minParam = configData.AINX_MIN?.additionParam;
  let maxParam = configData.AINX_MAX?.additionParam;
  minVal =
    configData.AINX_MIN != undefined && minParam != undefined
      ? args[minParam.toLowerCase()].value + configData.AINX_MIN[pga_gain]
      : minVal;
  maxVal =
    configData.AINX_MAX != undefined && maxParam != undefined
      ? args[maxParam.toLowerCase()].value + configData.AINX_MAX[pga_gain]
      : maxVal;
  return {
    absMinValue: absMinVal,
    absMaxValue: absMaxVal,
    removeTooltip: false,
    minValue: minVal,
    maxValue: maxVal,
    isDisabled: false,
    value: val,
  };
}

export function calculate_VIN_DIFF(args) {
  let pga_gain = baseCalculation.compute_pga_gain(args);
  const VIN_DIFF_MAX = this.get_VIN_DIFF_MAX(
    args.avdd.value,
    args.avss.value,
    pga_gain,
    args.vrefn.value,
    args.vrefp.value,
    args,
  );

  let updatedVinDiffConfig = {
    absMinValue: args.avss.value - 0.3,
    absMaxValue: args.avdd.value + 0.3,
    removeTooltip: false,
    isDisabled: false,
    minValue: -VIN_DIFF_MAX,
    maxValue: VIN_DIFF_MAX,
  };

  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData && configData.VREF_FSR_SF) {
    let VREF_FSR_SF = parseFloat(configData.VREF_FSR_SF);
    if (configData.controls != undefined) {
      VREF_FSR_SF = configData.controls.configuration.refSource
        ? configData.VREF_FSR_SF[
            configData.controls.configuration.refSource.value
          ]
        : parseFloat(configData.VREF_FSR_SF);
      if (configData.controls.configuration.refSource != undefined) {
        updatedVinDiffConfig["minValue"] = -(
          (VREF_FSR_SF / 2) *
          (args.vrefp.value / pga_gain)
        );
        updatedVinDiffConfig["maxValue"] =
          (VREF_FSR_SF / 2) * (args.vrefp.value / pga_gain);
      }
    }
  }

  if (args.input_mode.value === "Absolute Voltages") {
    updatedVinDiffConfig["value"] = args.ainp.value - args.ainn.value;
    updatedVinDiffConfig["absMinValue"] = "";
    updatedVinDiffConfig["absMaxValue"] = "";
    updatedVinDiffConfig["removeTooltip"] = true;
    updatedVinDiffConfig["isDisabled"] = true;
  }

  return updatedVinDiffConfig;
}

export function calculate_VIN_CM(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let pga_gain = baseCalculation.compute_pga_gain(args);
  let calculated_AGAIN =
    args.parameterConfigData[args.selectedDevice].AGAIN != undefined
      ? typeof args.parameterConfigData[args.selectedDevice].AGAIN === "object"
        ? args.parameterConfigData[args.selectedDevice].AGAIN[pga_gain]
        : parseInt(args.parameterConfigData[args.selectedDevice].AGAIN)
      : this.get_AGAIN(pga_gain);
  const modifiedVinDiff = Math.abs(
    (calculated_AGAIN * args.vin_diff.value) / 2,
  );

  let updatedVinCmConfig = {
    absMinValue: args.avss.value - 0.3,
    absMaxValue: args.avdd.value + 0.3,
    removeTooltip: false,
    isDisabled: false,
    minValue:
      this.get_VOUT_MIN(args.avss.value, pga_gain, args) + modifiedVinDiff,
    maxValue:
      this.get_VOUT_MAX(args.avdd.value, pga_gain, args) - modifiedVinDiff,
  };

  let absMinVal = baseCalculation.compute_AINx_ABS_MIN(args);
  let absMaxVal = baseCalculation.compute_AINx_ABS_MAX(args);
  let minVal;
  let maxVal;
  if (configData && configData.NEG_SWING_TO_RAIL != undefined) {
    if (configData.NEG_SWING_TO_RAIL.bufferEnabled != undefined) {
      minVal = args.pga_gain.value
        ? args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferEnabled)
        : args.avss.value +
          parseFloat(configData.NEG_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.NEG_SWING_TO_RAIL.bufferDisabled == undefined) {
      minVal =
        args.avss.value + parseFloat(configData.NEG_SWING_TO_RAIL[pga_gain]);
    }
  }
  if (configData && configData.POS_SWING_TO_RAIL != undefined) {
    if (configData.POS_SWING_TO_RAIL.bufferEnabled != undefined) {
      maxVal = args.pga_gain.value
        ? args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferEnabled)
        : args.avdd.value -
          parseFloat(configData.POS_SWING_TO_RAIL.bufferDisabled);
    }
    if (configData.POS_SWING_TO_RAIL.bufferDisabled == undefined) {
      maxVal =
        args.avdd.value - parseFloat(configData.POS_SWING_TO_RAIL[pga_gain]);
    }
  }

  let minParam = configData.AINX_MIN?.additionParam;
  let maxParam = configData.AINX_MAX?.additionParam;
  let ainxMin =
    configData.AINX_MIN != undefined
      ? args[minParam.toLowerCase()].value + configData.AINX_MIN[pga_gain]
      : absMinVal;
  let ainxMax =
    configData.AINX_MAX != undefined
      ? args[maxParam.toLowerCase()].value + configData.AINX_MAX[pga_gain]
      : absMaxVal;

  (updatedVinCmConfig["minValue"] = ainxMin + args.vin_diff.value / 2),
    (updatedVinCmConfig["maxValue"] = ainxMax - args.vin_diff.value / 2);

  if (args.input_mode.value === "Absolute Voltages") {
    updatedVinCmConfig["value"] = (args.ainp.value + args.ainn.value) / 2;
    updatedVinCmConfig["absMinValue"] = "";
    updatedVinCmConfig["absMaxValue"] = "";
    updatedVinCmConfig["removeTooltip"] = true;
    updatedVinCmConfig["isDisabled"] = true;
  }

  return updatedVinCmConfig;
}

//------- API to form Graph meta data ------//
export function computeCmRangeGraph(args) {
  let pga_gain = baseCalculation.compute_pga_gain(args);
  const vout_cm_clampped = this.get_VOUT_CM_CLAMPED(
    args.avdd.value,
    args.avss.value,
    pga_gain,
    args.vin_cm.value,
    args.vin_diff.value,
    args,
  );

  const vref_diff = this.get_VREF_DIFF(args.vrefn.value, args.vrefp.value);

  const pga_limits = this.get_PGA_INPUT_LIMITS(
    args.avdd.value,
    args.avss.value,
    pga_gain,
    args.vin_cm.maxValue,
    args.vin_cm.minValue,
    args.vin_diff.value,
    args.vrefn.value,
    args.vrefp.value,
    args,
  );

  return {
    metaData: {
      avdd: args.avdd.value,
      avss: args.avss.value,
      ainp: args.ainp.value,
      ainn: args.ainn.value,
      graph_image:
        args.parameterConfigData[args.selectedDevice].graph_image != undefined
          ? args.parameterConfigData[args.selectedDevice].graph_image
          : args.graph_image.value,

      voutp_clamped: this.get_VOUTP_CLAMPED(
        args.avdd.value,
        args.avss.value,
        pga_gain,
        args.vin_cm.value,
        args.vin_diff.value,
        args,
      ),
      voutn_clamped: this.get_VOUTN_CLAMPED(
        args.avdd.value,
        args.avss.value,
        pga_gain,
        args.vin_cm.value,
        args.vin_diff.value,
        args,
      ),
      vout_max: parseFloat(args.avdd.value - 0.1),
      vout_min: parseFloat(args.avss.value + 0.1),

      vref_positive_limit: vout_cm_clampped + vref_diff / (2 * DGAIN),
      vref_negative_limit: vout_cm_clampped - vref_diff / (2 * DGAIN),

      pga_input_pos_limit: pga_limits.posLimit,
      pga_input_neg_limit: pga_limits.negLimit,

      error_voutp: {
        status: this.check_ERROR_VOUTP(
          args.avdd.value,
          args.avss.value,
          pga_gain,
          args.vin_cm.value,
          args.vin_diff.value,
          args,
        ),
        text: "VOUTP is saturated",
      },

      error_voutn: {
        status: this.check_ERROR_VOUTN(
          args.avdd.value,
          args.avss.value,
          pga_gain,
          args.vin_cm.value,
          args.vin_diff.value,
          args,
        ),
        text: "VOUTN is saturated",
      },

      error_vout_diff_exceeds_fsr: {
        status: this.check_ERROR_VOUT_DIFF_EXCEEDS_FSR(
          args.avdd.value,
          args.avss.value,
          pga_gain,
          args.vin_cm.value,
          args.vin_diff.value,
          args.vrefn.value,
          args.vrefp.value,
          args,
        ),
        text: "Differential output voltage too large<br>(output codes clipped)",
      },
    },
  };
}

// ---------- CM calculator's other parameter calculations ---------------//
export function get_BUFFER_ENABLED(GAIN) {
  return baseCalculation.get_BUFFER_ENABLED(GAIN);
}

export function get_GAIN_LIST() {
  return baseCalculation.get_GAIN_LIST;
}

export function get_AGAIN(GAIN) {
  return baseCalculation.get_AGAIN(GAIN);
}

export function get_DGAIN(GAIN) {
  return baseCalculation.get_DGAIN(GAIN);
}

export function getEquivalentGain(GAIN, args) {
  return baseCalculation.getEquivalentGain(GAIN, args);
}

export function get_VREF_FSR_SF() {
  return baseCalculation.get_VREF_FSR_SF;
}

export function get_POS_SWING_TO_RAIL(isBufferEnabled) {
  return baseCalculation.get_POS_SWING_TO_RAIL(isBufferEnabled);
}

export function get_NEG_SWING_TO_RAIL(isBufferEnabled) {
  return baseCalculation.get_NEG_SWING_TO_RAIL(isBufferEnabled);
}

export function get_AINx_ABS_MIN(AVSS) {
  return baseCalculation.get_AINx_ABS_MIN(AVSS);
}

export function get_AINx_ABS_MAX(AVDD) {
  return baseCalculation.get_AINx_ABS_MAX(AVDD);
}

export function get_AVSS_MIN() {
  return baseCalculation.get_AVSS_MIN;
}

export function get_AVSS_MAX() {
  return baseCalculation.get_AVSS_MAX;
}

export function get_ASUPPLY_MIN() {
  return baseCalculation.get_ASUPPLY_MIN;
}

export function get_ASUPPLY_MAX() {
  return baseCalculation.get_ASUPPLY_MAX;
}

export function get_VREF_DIFF_MIN() {
  return baseCalculation.get_VREF_DIFF_MIN;
}

export function get_VREF_DIFF_MAX(AVDD, AVSS) {
  return baseCalculation.get_VREF_DIFF_MAX(AVDD, AVSS);
}

export function get_VREFP_TO_AVDD_MAX(GAIN) {
  return baseCalculation.get_VREFP_TO_AVDD_MAX(GAIN);
}

export function get_VREFN_TO_AVSS_MAX(GAIN) {
  return baseCalculation.get_VREFN_TO_AVSS_MAX(GAIN);
}

// ----------- Other background calculaions ---------- //
export function get_VREF_DIFF(VREFN, VREFP) {
  return baseCalculation.get_VREF_DIFF(VREFN, VREFP);
}

export function get_FSR(GAIN, VREFN, VREFP, args) {
  return baseCalculation.get_FSR(GAIN, VREFN, VREFP, args);
}

export function get_ASUPPLY(AVDD, AVSS) {
  return baseCalculation.get_ASUPPLY(AVDD, AVSS);
}

export function get_PGA_OUTPUT_SWING_MAX(ASUPPLY, GAIN, args) {
  return baseCalculation.get_PGA_OUTPUT_SWING_MAX(ASUPPLY, GAIN, args);
}

export function get_VIN_DIFF_MAX(AVDD, AVSS, GAIN, VREFN, VREFP, args) {
  return this.get_FSR(GAIN, VREFN, VREFP, args) / 2;
}

export function get_VOUTP_IDEAL(GAIN, VIN_CM, VIN_DIFF, args) {
  return baseCalculation.get_VOUTP_IDEAL(GAIN, VIN_CM, VIN_DIFF, args);
}

export function get_VOUTN_IDEAL(GAIN, VIN_CM, VIN_DIFF, args) {
  return baseCalculation.get_VOUTN_IDEAL(GAIN, VIN_CM, VIN_DIFF, args);
}

export function get_VOUT_DIFF_IDEAL(GAIN, VIN_CM, VIN_DIFF, args) {
  return baseCalculation.get_VOUT_DIFF_IDEAL(GAIN, VIN_CM, VIN_DIFF, args);
}

export function get_VOUT_MAX(AVDD, GAIN, args) {
  return baseCalculation.get_VOUT_MAX(AVDD, GAIN, args);
}

export function get_VOUT_MIN(AVSS, GAIN, args) {
  return baseCalculation.get_VOUT_MIN(AVSS, GAIN, args);
}

export function get_VOUTP_CLAMPED(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args) {
  return (
    (GAIN / 2) * (args.ainp.value - args.ainn.value) +
    this.get_VOUT_CM_CLAMPED(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args)
  );
}

export function get_VOUTN_CLAMPED(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args) {
  return (
    (GAIN / 2) * (args.ainn.value - args.ainp.value) +
    this.get_VOUT_CM_CLAMPED(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args)
  );
}

export function get_VOUT_DIFF_CLAMPED(
  AVDD,
  AVSS,
  GAIN,
  VIN_CM,
  VIN_DIFF,
  args,
) {
  return baseCalculation.get_VOUT_DIFF_CLAMPED(
    AVDD,
    AVSS,
    GAIN,
    VIN_CM,
    VIN_DIFF,
    args,
  );
}

export function get_VOUT_CM_CLAMPED(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args) {
  return args.avdd.value / 2;
}

export function get_PGA_INPUT_LIMITS(
  AVDD,
  AVSS,
  GAIN,
  VCM_MAX,
  VCM_MIN,
  VIN_DIFF,
  VREFN,
  VREFP,
  args,
) {
  const VIN_DIFF_MAX = this.get_VIN_DIFF_MAX(
    AVDD,
    AVSS,
    GAIN,
    VREFN,
    VREFP,
    args,
  );

  if (Math.abs(VIN_DIFF) > VIN_DIFF_MAX) {
    VCM_MAX = this.get_ASUPPLY(AVDD, AVSS) / 2;
    VCM_MIN = VCM_MAX;
  }
  let configData = args.parameterConfigData[args.selectedDevice];
  let minParam = configData.AINX_MIN?.additionParam;
  let maxParam = configData.AINX_MAX?.additionParam;
  let negLimitValue =
    configData.AINX_MIN != undefined && minParam != undefined
      ? args[minParam.toLowerCase()].value + configData.AINX_MIN[GAIN]
      : this.get_VOUT_MIN(AVDD, GAIN, args);
  let posLimitValue =
    configData.AINX_MAX != undefined && maxParam != undefined
      ? args[maxParam.toLowerCase()].value + configData.AINX_MAX[GAIN]
      : this.get_VOUT_MAX(AVSS, GAIN, args);
  return {
    posLimit: posLimitValue,
    negLimit: negLimitValue,
  };
}

// ------------ Voltage limits for analog inputs ---------//
export function getValueForClip_AINP(condition, AINN, VIN_DIFF_MAX) {
  return baseCalculation.getValueForClip_AINP(condition, AINN, VIN_DIFF_MAX);
}

export function getValueForClip_AINN(condition, AINP, VIN_DIFF_MAX) {
  return baseCalculation.getValueForClip_AINN(condition, AINN, VIN_DIFF_MAX);
}

export function g(GAIN, x, y, args) {
  return baseCalculation.g(GAIN, x, y, args);
}

export function h(GAIN, x, y, args) {
  return baseCalculation.h(GAIN, x, y, args);
}

export function get_AINP_VOUTP_POS_SAT(AINN, AVDD, GAIN, args) {
  return baseCalculation.get_AINN_VOUTP_POS_SAT(AINN, AVDD, GAIN, args);
}

export function get_AINP_VOUTP_NEG_SAT(AINN, AVSS, GAIN, args) {
  return baseCalculation.get_AINP_VOUTP_NEG_SAT(AINN, AVSS, GAIN, args);
}

export function get_AINP_VOUTN_POS_SAT(AINN, AVDD, GAIN, args) {
  return baseCalculation.get_AINP_VOUTN_POS_SAT(AINN, AVDD, GAIN, args);
}

export function get_AINP_VOUTN_NEG_SAT(AINN, AVSS, GAIN, args) {
  return baseCalculation.get_AINP_VOUTN_NEG_SAT(AINN, AVSS, GAIN, args);
}

export function get_AINN_VOUTP_POS_SAT(AINP, AVDD, GAIN, args) {
  return baseCalculation.get_AINN_VOUTP_POS_SAT(AINP, AVDD, GAIN, args);
}

export function get_AINN_VOUTP_NEG_SAT(AINP, AVSS, GAIN, args) {
  return baseCalculation.get_AINN_VOUTP_NEG_SAT(AINP, AVSS, GAIN, args);
}

export function get_AINN_VOUTN_POS_SAT(AINP, AVDD, GAIN, args) {
  return baseCalculation.get_AINN_VOUTN_POS_SAT(AINP, AVDD, GAIN, args);
}

export function get_AINN_VOUTN_NEG_SAT(AINP, AVSS, GAIN, args) {
  return baseCalculation.get_AINN_VOUTN_NEG_SAT(AINP, AVSS, GAIN, args);
}

// ---------- Error indicators ---------- //
export function check_ERROR_VOUTP(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args) {
  return false;
}

export function check_ERROR_VOUTN(AVDD, AVSS, GAIN, VIN_CM, VIN_DIFF, args) {
  return false;
}

export function check_ERROR_VOUT_DIFF_EXCEEDS_FSR(
  AVDD,
  AVSS,
  GAIN,
  VIN_CM,
  VIN_DIFF,
  VREFN,
  VREFP,
  args,
) {
  return baseCalculation.check_ERROR_VOUT_DIFF_EXCEEDS_FSR(
    AVDD,
    AVSS,
    GAIN,
    VIN_CM,
    VIN_DIFF,
    VREFN,
    VREFP,
    args,
  );
}

function get_Control_Object(controlData) {
  if (typeof controlData === "string" && controlData.includes("|")) {
    // Split the string by pipe characters (|)
    const valueArray = controlData.split("|");

    // Find the index of the element containing "(default)"
    const defaultValueIndex = valueArray.findIndex(element =>
      element.includes("(default)"),
    );

    // Map the array values to an object with indices
    const controlObject = {
      options: valueArray.reduce((acc, value, index) => {
        acc[index] = value.replace("(default)", "").trim();
        return acc;
      }, {}),
      value: defaultValueIndex !== -1 ? defaultValueIndex : 0,
    };

    return controlObject;
  }
  // Return an empty object if the input is not a string or object
  return {};
}
